/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <catch2/catch_test_macros.hpp>

#include "aidge/operator/MetaOperator.hpp"
#include "aidge/operator/MetaOperatorDefs.hpp"
#include "aidge/graph/GraphView.hpp"
#include "aidge/graph/Testing.hpp"
#include <cstddef>

using namespace Aidge;

TEST_CASE("[core/operators] MetaOperator", "[Operator]") {
    SECTION("PaddedConv") {
        auto op = PaddedConv(1, 3, {3, 3}, "padded_conv", {1, 1}, {1, 1, 1, 1});

        auto microGraph = std::dynamic_pointer_cast<MetaOperator_Op>(op->getOperator())->getMicroGraph();

        REQUIRE(microGraph->getNodes().size() == 2);
        REQUIRE(microGraph->inputNodes().size() == 2);  // 2 because Conv has inputs outside the meta-op (Producers for weight and bias)
        REQUIRE(nodePtrTo(microGraph->getOrderedInputs()) == std::vector<std::pair<std::string, IOIndex_t>>({{"Pad", 0}, {"Conv", 1}, {"Conv", 2}}));
        REQUIRE(nodePtrTo(microGraph->getOrderedOutputs()) == std::vector<std::pair<std::string, IOIndex_t>>({{"Conv", 0}}));
        REQUIRE(microGraph->outputNodes().size() == 1);
        REQUIRE((*microGraph->outputNodes().begin())->getOperator()->type() == "Conv");
        REQUIRE(op->nbInputs() == 3);
        REQUIRE(op->nbData() == 1);
        REQUIRE(op->nbOutputs() == 1);

        std::shared_ptr<Tensor> myInput = std::make_shared<Tensor>();
        myInput->resize({2,3,5,5});
        std::shared_ptr<OperatorTensor> opTensor = std::static_pointer_cast<OperatorTensor>(op->getOperator());
        opTensor->associateInput(0,myInput);
        opTensor->computeOutputDims();

        REQUIRE(opTensor->outputDimsForwarded());
        REQUIRE(std::static_pointer_cast<Tensor>(opTensor->getRawOutput(0))->dims() == std::vector<size_t>({2,3,5,5}));
        REQUIRE(std::static_pointer_cast<Tensor>(opTensor->getRawInput(0)) == myInput);
        REQUIRE(microGraph->getOrderedInputs()[0].first->getOperator()->getRawInput(0) == myInput);
        REQUIRE(opTensor->getRawOutput(0) == (*microGraph->outputNodes().begin())->getOperator()->getRawOutput(0));

        //op->getOperator()->updateConsummerProducer();  // require implementation
        //auto microGraphScheduler = std::dynamic_pointer_cast<MetaOperator_Op>(op->getOperator())->getMicroGraphScheduler();
        //REQUIRE(microGraphScheduler->getStaticScheduling().size() == 2);
    }
}
